@@ -72,6 +72,8 @@ typedef struct {
 #define WMFHideOtherApplications		10
 #define WMFHideApplication			12
 
+#define GSMaxWMProtocols 5
+
 /* Graphics Driver protocol. Setup in [NSGraphicsContext-contextDevice:] */
 enum {
   GDriverHandlesBacking = 1,
@@ -109,12 +111,17 @@ typedef struct _gswindow_device_t {
   int			xoff;
   int			yoff;
   int			boff;
-  Atom			protocols[4];
+  Atom			protocols[GSMaxWMProtocols];
   int			numProtocols;
   XIC                   ic;
   void                  *gdriver;      /* gdriver ident. Managed by gdriver */
   int                   gdriverProtocol; /* Managed by gdriver */
   BOOL			ignore_take_focus;
+#ifdef HAVE_LIBXEXT
+  uint32_t              net_wm_sync_request_counter_value_low;
+  uint32_t              net_wm_sync_request_counter_value_high;
+  XSyncCounter          net_wm_sync_request_counter;
+#endif
 } gswindow_device_t;
 
 #define GET_XDRAWABLE(win)  ((win)->buffer ? (win)->buffer: (win)->ident)
