@@ -52,6 +52,20 @@ not just discard the extra bits
   bit_ofs %= 8;
 
   v = 0;
+
+  /* if we are handling 16 bit values we optimize */
+
+
+  if (num_bits == 16)
+    {
+#if (GS_WORDS_BIGENDIAN==0)
+      ptr++;
+#endif 
+      v = *ptr;
+      return v;
+    }
+
+  
   for (i = 0; i < 8 && i < num_bits; i++)
     {
       v <<= 1;
