@@ -31,7 +31,6 @@
 #include <AppKit/NSPrintOperation.h>
 
 #include "cairo/CairoContext.h"
-#include "cairo/CairoGState.h"
 #include "cairo/CairoSurface.h"
 #include "cairo/CairoPSSurface.h"
 #include "cairo/CairoPDFSurface.h"
@@ -42,7 +41,9 @@
 #define CGSTATE ((CairoGState *)gstate)
 
 #if BUILD_SERVER == SERVER_x11
+#  include "cairo/CairoGState.h"
 #  include "x11/XGServer.h"
+#  define _CAIRO_GSTATE_CLASSNAME CairoGState
 #  ifdef USE_GLITZ
 #    define _CAIRO_SURFACE_CLASSNAME XGCairoGlitzSurface
 #    include "cairo/XGCairoGlitzSurface.h"
@@ -57,7 +58,9 @@
 #  include "x11/XGServerWindow.h"
 #  include "x11/XWindowBuffer.h"
 #elif BUILD_SERVER == SERVER_win32
+#  include "cairo/Win32CairoGState.h"
 #  include <windows.h>
+#  define _CAIRO_GSTATE_CLASSNAME Win32CairoGState
 #  ifdef USE_GLITZ
 #    define _CAIRO_SURFACE_CLASSNAME Win32CairoGlitzSurface
 #    include "cairo/Win32CairoGlitzSurface.h"
@@ -69,17 +72,6 @@
 #  error Invalid server for Cairo backend : non implemented
 #endif /* BUILD_SERVER */
 
-@interface NSBitmapImageRep (GSPrivate)
-- (NSBitmapImageRep *) _convertToFormatBitsPerSample: (int)bps
-                                     samplesPerPixel: (int)spp
-                                            hasAlpha: (BOOL)alpha
-                                            isPlanar: (BOOL)isPlanar
-                                      colorSpaceName: (NSString*)colorSpaceName
-                                        bitmapFormat: (NSBitmapFormat)bitmapFormat 
-                                         bytesPerRow: (int)rowBytes
-                                        bitsPerPixel: (int)pixelBits;
-@end
-
 @implementation CairoContext
 
 + (void) initializeBackend
@@ -92,7 +84,7 @@
 
 + (Class) GStateClass
 {
-  return [CairoGState class];
+  return [_CAIRO_GSTATE_CLASSNAME class];
 }
 
 + (BOOL) handlesPS
@@ -105,31 +97,50 @@
   return YES;
 }
 
+- (BOOL) isDrawingToScreen
+{
+  CairoSurface *surface = nil;
+  [CGSTATE GSCurrentSurface: &surface : NULL : NULL];
+  return [surface isDrawingToScreen];
+}
+
 - (void) flushGraphics
 {
   // FIXME: Why is this here? When is it called?
 #if BUILD_SERVER == SERVER_x11
   XFlush([(XGServer *)server xDisplay]);
+#elif BUILD_SERVER == SERVER_win32
+  CairoSurface *surface = nil;
+
+  [CGSTATE GSCurrentSurface: &surface : NULL : NULL];
+  if ((surface != nil) && ([surface surface] != NULL))
+    {
+      cairo_surface_flush([surface surface]);
+    }
 #endif // BUILD_SERVER = SERVER_x11
 }
 
-#if BUILD_SERVER == SERVER_x11
 
 /* Private backend methods */
 + (void) handleExposeRect: (NSRect)rect forDriver: (void *)driver
 {
+#if BUILD_SERVER == SERVER_x11
   if ([(id)driver isKindOfClass: [XWindowBuffer class]])
     {
       // For XGCairoXImageSurface
       [(XWindowBuffer *)driver _exposeRect: rect];
     }
-  else if ([(id)driver isKindOfClass: [CairoSurface class]])
+  else
+#endif
+  if ([(id)driver isKindOfClass: [CairoSurface class]])
     {
       // For XGCairoModernSurface
       [(CairoSurface *)driver handleExposeRect: rect];
     }
 }
 
+#if BUILD_SERVER == SERVER_x11
+
 #ifdef XSHM
 
 + (void) _gotShmCompletion: (Drawable)d
@@ -150,48 +161,35 @@
 
 @implementation CairoContext (Ops) 
 
-- (void) GSDrawImage: (NSRect)rect: (void *)imageref
+- (BOOL) isCompatibleBitmap: (NSBitmapImageRep*)bitmap
 {
-  NSBitmapImageRep *bitmap;
-  const unsigned char *data[5];
   NSString *colorSpaceName;
 
-  bitmap = (NSBitmapImageRep*)imageref;
-  colorSpaceName = [bitmap colorSpaceName];
-  if ([bitmap isPlanar] || ([bitmap bitmapFormat] != 0) 
-      || ([bitmap bitsPerSample] != 8) ||
-      (![colorSpaceName isEqualToString: NSDeviceRGBColorSpace] &&
-       ![colorSpaceName isEqualToString: NSCalibratedRGBColorSpace]))
+  if ([bitmap bitmapFormat] != 0)
     {
-      int bitsPerSample = 8;
-      BOOL isPlanar = NO;
-      int samplesPerPixel = [bitmap hasAlpha] ? 4 : 3;
-      NSString *colorSpaceName = NSCalibratedRGBColorSpace;
-      NSBitmapImageRep *new;
-
-     new = [bitmap _convertToFormatBitsPerSample: bitsPerSample
-                    samplesPerPixel: samplesPerPixel
-                    hasAlpha: [bitmap hasAlpha]
-                    isPlanar: isPlanar
-                    colorSpaceName: colorSpaceName
-                    bitmapFormat: 0
-                    bytesPerRow: 0
-                    bitsPerPixel: 0];
-            
-      if (new == nil)
-        {
-          NSLog(@"Could not convert bitmap data");
-          return;
-        }
-      bitmap = new;
+      return NO;
     }
 
-  [bitmap getBitmapDataPlanes: (unsigned char **)&data];
-  [self NSDrawBitmap: rect : [bitmap pixelsWide] : [bitmap pixelsHigh]
-        : [bitmap bitsPerSample] : [bitmap samplesPerPixel]
-        : [bitmap bitsPerPixel] : [bitmap bytesPerRow] : [bitmap isPlanar]
-        : [bitmap hasAlpha] :  [bitmap colorSpaceName]
-        : data];
+  if ([bitmap isPlanar])
+    {
+      return NO;
+    }
+
+  if ([bitmap bitsPerSample] != 8)
+    {
+      return NO;
+    }
+
+  colorSpaceName = [bitmap colorSpaceName];
+  if (![colorSpaceName isEqualToString: NSDeviceRGBColorSpace] &&
+      ![colorSpaceName isEqualToString: NSCalibratedRGBColorSpace])
+    {
+      return NO;
+    }
+  else
+    {
+      return YES;
+    }
 }
 
 - (void) GSCurrentDevice: (void **)device : (int *)x : (int *)y
@@ -285,3 +283,5 @@
 @end
 
 #undef _CAIRO_SURFACE_CLASSNAME
+#undef _CAIRO_GSTATE_CLASSNAME
+
