@@ -37,29 +37,6 @@
 
 @implementation CairoFontInfo 
 
-- (void) setCacheSize: (unsigned int)size
-{
-  _cacheSize = size;
-  if (_cachedSizes)
-    {
-      free(_cachedSizes);
-    }
-  if (_cachedGlyphs)
-    {
-      free(_cachedGlyphs);
-    }
-  _cachedSizes = malloc(sizeof(NSSize) * size);
-  if (_cachedSizes)
-    {
-      memset(_cachedSizes, 0, sizeof(NSSize) * size);
-    }
-  _cachedGlyphs = malloc(sizeof(unsigned int) * size);
-  if (_cachedGlyphs)
-    {
-      memset(_cachedGlyphs, 0, sizeof(unsigned int) * size);
-    }
-}
-
 - (BOOL) setupAttributes
 {
   cairo_font_extents_t font_extents;
@@ -68,32 +45,15 @@
   cairo_matrix_t ctm;
   cairo_font_options_t *options;
 
-  ASSIGN(_faceInfo, [CairoFontEnumerator fontWithName: fontName]);
-  if (!_faceInfo)
+  if (![super setupAttributes])
     {
       return NO;
     }
 
-  // check for font specific cache size from face info
-  [self setCacheSize: [_faceInfo cacheSize]];
-
-  /* setting GSFontInfo:
-   * weight, traits, familyName,
-   * mostCompatibleStringEncoding, encodingScheme, coveredCharacterSet
-   */
-
-  weight = [_faceInfo weight];
-  traits = [_faceInfo traits];
-  familyName = [[_faceInfo familyName] copy];
-  mostCompatibleStringEncoding = NSUTF8StringEncoding;
-  encodingScheme = @"iso10646-1";
-  coveredCharacterSet = [[_faceInfo characterSet] retain];
-
-
   /* setting GSFontInfo:
    * xHeight, pix_width, pix_height
    */
-  cairo_matrix_init(&font_matrix, matrix[0], matrix[1], matrix[2],
+  cairo_matrix_init(&font_matrix, matrix[0], matrix[1], -matrix[2],
                     matrix[3], matrix[4], matrix[5]);
   //cairo_matrix_scale(&font_matrix, 0.9, 0.9);
   cairo_matrix_init_identity(&ctm);
@@ -190,23 +150,13 @@
 
 - (void) dealloc
 {
-  RELEASE(_faceInfo);
   if (_scaled)
     {
       cairo_scaled_font_destroy(_scaled);
     }
-  if (_cachedSizes)
-    free(_cachedSizes);
-  if (_cachedGlyphs)
-    free(_cachedGlyphs);
   [super dealloc];
 }
 
-- (CGFloat) defaultLineHeightForFont
-{
-  return lineHeight;
-}
-
 - (BOOL) glyphIsEncoded: (NSGlyph)glyph
 {
   /* FIXME: There is no proper way to determine with the toy font API,
@@ -309,15 +259,6 @@ BOOL _cairo_extents_for_NSGlyph(cairo_scaled_font_t *scaled_font, NSGlyph glyph,
   return 0.0;
 }
 
-- (NSGlyph) glyphWithName: (NSString *) glyphName
-{
-  /* subclass should override */
-  /* terrible! FIXME */
-  NSGlyph g = [glyphName cString][0];
-
-  return g;
-}
-
 - (void) appendBezierPathWithGlyphs: (NSGlyph *)glyphs 
                               count: (int)length 
                        toBezierPath: (NSBezierPath *)path
@@ -494,7 +435,7 @@ BOOL _cairo_extents_for_NSGlyph(cairo_scaled_font_t *scaled_font, NSGlyph glyph,
       return;
     }
 
-  cairo_matrix_init(&font_matrix, matrix[0], matrix[1], matrix[2],
+  cairo_matrix_init(&font_matrix, matrix[0], matrix[1], -matrix[2],
                     matrix[3], matrix[4], matrix[5]);
   cairo_set_font_matrix(ct, &font_matrix);
   if (cairo_status(ct) != CAIRO_STATUS_SUCCESS)
