@@ -1,3 +1,4 @@
+
 /*
    CairoGState.m
 
@@ -73,7 +74,7 @@
     NSZoneFree(NSDefaultMallocZone(), _base); \
   }
 
-static inline double doubleFromUserSpace(NSAffineTransform *ctm, double d)
+static inline double doubleFromUserSpace(NSAffineTransform *ctm, CGFloat d)
 {
   if (ctm)
     {
@@ -86,7 +87,7 @@ static inline double doubleFromUserSpace(NSAffineTransform *ctm, double d)
   return d;
 }
 
-static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
+static inline CGFloat floatToUserSpace(NSAffineTransform *ctm, double d)
 {
   if (ctm)
     {
@@ -97,7 +98,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
       d = doubleFromUserSpace(ictm, d);
       RELEASE(ictm);
     }
-  return (float)d;
+  return (CGFloat)d;
 }
 
 
@@ -122,6 +123,14 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
   [super dealloc];
 }
 
+- (NSString*) description
+{
+  NSMutableString *description = [[[super description] mutableCopy] autorelease];
+  [description appendFormat: @" surface: %@",_surface];
+  [description appendFormat: @" context: %p",_ct];
+  return [[description copy] autorelease];
+}
+
 - (id) copyWithZone: (NSZone *)zone
 {
   CairoGState *copy = (CairoGState *)[super copyWithZone: zone];
@@ -258,7 +267,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
   return copy;
 }
 
-- (void) GSCurrentSurface: (CairoSurface **)surface: (int *)x : (int *)y
+- (void) GSCurrentSurface: (CairoSurface **)surface : (int *)x : (int *)y
 {
   if (x)
     *x = offset.x;
@@ -407,7 +416,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
     }
 }
 
-- (void) GSSetFontSize: (float)size
+- (void) GSSetFontSize: (CGFloat)size
 {
   if (_ct)
     {
@@ -491,6 +500,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
   if (_ct)
     {
       cairo_destroy(_ct);
+      _ct = NULL;
     }
   if (!_surface)
     {
@@ -519,11 +529,11 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
   cairo_new_path(_ct);
 }
 
-- (void) DPScurrentflat: (float *)flatness
+- (void) DPScurrentflat: (CGFloat *)flatness
 {
   if (_ct)
     {
-      *flatness = (float)cairo_get_tolerance(_ct) * 2;
+      *flatness = (CGFloat)cairo_get_tolerance(_ct) * 2;
     }
 }
 
@@ -583,7 +593,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
    */
 }
 
-- (void) DPScurrentlinewidth: (float *)width
+- (void) DPScurrentlinewidth: (CGFloat *)width
 {
   if (_ct)
     {
@@ -591,7 +601,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
     }
 }
 
-- (void) DPScurrentmiterlimit: (float *)limit
+- (void) DPScurrentmiterlimit: (CGFloat *)limit
 {
   if (_ct)
     {
@@ -603,7 +613,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
 {
 }
 
-- (void) DPSsetdash: (const float *)pat : (int)size : (float)foffset
+- (void) DPSsetdash: (const CGFloat *)pat : (NSInteger)size : (CGFloat)foffset
 {
   if (_ct)
     {
@@ -623,7 +633,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
     }
 }
 
-- (void) DPSsetflat: (float)flatness
+- (void) DPSsetflat: (CGFloat)flatness
 {
   [super DPSsetflat: flatness];
   if (_ct)
@@ -650,7 +660,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
     }
 }
 
-- (void) DPSsetlinewidth: (float)width
+- (void) DPSsetlinewidth: (CGFloat)width
 {
   if (_ct)
     {
@@ -658,7 +668,7 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
     }
 }
 
-- (void) DPSsetmiterlimit: (float)limit
+- (void) DPSsetmiterlimit: (CGFloat)limit
 {
   if (_ct)
     {
@@ -676,10 +686,11 @@ static inline float floatToUserSpace(NSAffineTransform *ctm, double d)
 
 - (void) _setPath
 {
-  unsigned count = [path elementCount];
-  unsigned i;
+  NSInteger count = [path elementCount];
+  NSInteger i;
   SEL elmsel = @selector(elementAtIndex:associatedPoints:);
-  IMP elmidx = [path methodForSelector: elmsel];
+  NSBezierPathElement (*elmidx)(id, SEL, NSInteger, NSPoint*) =
+    (NSBezierPathElement (*)(id, SEL, NSInteger, NSPoint*))[path methodForSelector: elmsel];
 
   // reset current cairo path
   cairo_new_path(_ct);
@@ -992,10 +1003,10 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
   cairo_fill(_ct);
 }
 
-- (void) DPSimage: (NSAffineTransform *)matrix : (int)pixelsWide
-		 : (int)pixelsHigh : (int)bitsPerSample 
-		 : (int)samplesPerPixel : (int)bitsPerPixel
-		 : (int)bytesPerRow : (BOOL)isPlanar
+- (void) DPSimage: (NSAffineTransform *)matrix : (NSInteger)pixelsWide
+		 : (NSInteger)pixelsHigh : (NSInteger)bitsPerSample 
+		 : (NSInteger)samplesPerPixel : (NSInteger)bitsPerPixel
+		 : (NSInteger)bytesPerRow : (BOOL)isPlanar
 		 : (BOOL)hasAlpha : (NSString *)colorSpaceName
 		 : (const unsigned char *const[5])data
 {
@@ -1159,6 +1170,7 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
     cairo_matrix_init_scale(&source_matrix, 1, -1);
     cairo_matrix_translate(&source_matrix, 0, -pixelsHigh);
     cairo_pattern_set_matrix(cpattern, &source_matrix);
+    cairo_pattern_set_filter(cpattern, CAIRO_FILTER_BILINEAR);
     if (cairo_version() >= CAIRO_VERSION_ENCODE(1, 6, 0))
       {
         cairo_pattern_set_extend(cpattern, CAIRO_EXTEND_PAD);
@@ -1188,12 +1200,24 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
       device_color_t c;
 
       cairo_save(_ct);
-      _set_op(_ct, op);
 
-      c = fillColor;
-      gsColorToRGB(&c);
-      // The underlying concept does not allow to determine if alpha is set or not.
-      cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 9, 4)
+      if (GSCompositeHighlight == op)
+        {
+          cairo_set_operator(_ct, CAIRO_OPERATOR_DIFFERENCE);
+          cairo_set_source_rgb(_ct, 1, 1, 1);
+        }
+      else
+#endif
+        {
+          _set_op(_ct, op);
+          
+          c = fillColor;
+          gsColorToRGB(&c);
+          // The underlying concept does not allow to determine if alpha is set or not.
+          cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
+        }
+
       // This is almost a rectclip::::, but the path stays unchanged.
       path = [NSBezierPath bezierPathWithRect: aRect];
       [path transformUsingAffineTransform: ctm];
@@ -1209,7 +1233,7 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
                 fromRect: (NSRect)srcRect 
                  toPoint: (NSPoint)destPoint 
                       op: (NSCompositingOperation)op
-                fraction: (float)delta
+                fraction: (CGFloat)delta
 {
   cairo_surface_t *src;
   NSSize ssize = NSZeroSize;
@@ -1237,6 +1261,8 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
   cairo_pattern_t *cpattern;
   cairo_matrix_t source_matrix;
 
+  NSDebugMLLog(@"CairoGState", @"self: %@\n", self);
+
   if (!_ct || !source->_ct)
     {
       return;
@@ -1319,6 +1345,7 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
   //cairo_matrix_translate(&source_matrix, 0,  -[_surface size].height);
   cairo_matrix_translate(&source_matrix, minx - x + dx, miny - y + dy - ssize.height);
   cairo_pattern_set_matrix(cpattern, &source_matrix);
+  cairo_pattern_set_filter(cpattern, CAIRO_FILTER_BILINEAR);
   cairo_set_source(_ct, cpattern);
   cairo_pattern_destroy(cpattern);
   cairo_rectangle(_ct, x, y, width, height);
@@ -1348,7 +1375,7 @@ doesn't support to use the receiver cairo target as the source. */
            fromRect: (NSRect)aRect 
             toPoint: (NSPoint)aPoint 
                  op: (NSCompositingOperation)op
-           fraction: (float)delta
+           fraction: (CGFloat)delta
 {
   NSAffineTransformStruct tstruct =  [ctm transformStruct];
   cairo_surface_t *src;
@@ -1358,6 +1385,11 @@ doesn't support to use the receiver cairo target as the source. */
   cairo_matrix_t local_matrix;
   cairo_matrix_t source_matrix;
 
+  NSDebugMLLog(@"CairoGState", @"source: %@ fromRect: %@ toPoint: %@\n",
+               source,
+               NSStringFromRect(aRect),
+               NSStringFromPoint(aPoint));
+  
   if (!_ct || !source->_ct)
     {
       NSLog(@"WARNING: -drawGState called with a NULL target context (%p) or source context (%p)",
@@ -1405,6 +1437,7 @@ doesn't support to use the receiver cairo target as the source. */
   cairo_matrix_init_scale(&source_matrix, 1, -1);
   cairo_matrix_translate(&source_matrix, 0, -[source->_surface size].height);
   cairo_pattern_set_matrix(cpattern, &source_matrix);
+  cairo_pattern_set_filter(cpattern, CAIRO_FILTER_BILINEAR);
   if (cairo_version() >= CAIRO_VERSION_ENCODE(1, 6, 0))
     {
       cairo_pattern_set_extend(cpattern, CAIRO_EXTEND_PAD);
@@ -1575,16 +1608,39 @@ doesn't support to use the receiver cairo target as the source. */
           double green;
           double blue;
           double alpha;
-          
+          NSString *colorSpaceName;
+	  
           [gradient getColor: &color
                     location: &location
                      atIndex: i];
-          red = [color redComponent];
-          green = [color greenComponent];
-          blue = [color blueComponent];
-          alpha = [color alphaComponent];
-          cairo_pattern_add_color_stop_rgba(cpattern, location,
-                                            red, green, blue, alpha);
+
+	  colorSpaceName = [color colorSpaceName];
+	  if([NSCalibratedRGBColorSpace isEqualToString: colorSpaceName] ||
+	     [NSDeviceRGBColorSpace isEqualToString: colorSpaceName])
+	    {
+	      red = [color redComponent];
+	      green = [color greenComponent];
+	      blue = [color blueComponent];
+	      alpha = [color alphaComponent];
+	      cairo_pattern_add_color_stop_rgba(cpattern, location,
+						red, green, blue, alpha);
+	    }
+	  else if([NSCalibratedWhiteColorSpace isEqualToString: colorSpaceName] ||
+		  [NSDeviceWhiteColorSpace isEqualToString: colorSpaceName] ||
+		  [NSCalibratedBlackColorSpace isEqualToString: colorSpaceName] ||
+		  [NSDeviceBlackColorSpace isEqualToString: colorSpaceName])
+	    {
+	      red = [color whiteComponent];
+	      green = [color whiteComponent];
+	      blue = [color whiteComponent];
+	      alpha = [color alphaComponent];
+	      cairo_pattern_add_color_stop_rgba(cpattern, location,
+						red, green, blue, alpha);
+	    }
+	  else
+	    {
+	      NSLog(@"Cannot draw gradient for %@",colorSpaceName); 
+	    }
         }
       
       cairo_save(_ct);
