@@ -29,6 +29,7 @@
 #include <AppKit/NSAffineTransform.h>
 #include <AppKit/NSBitmapImageRep.h>
 #include <AppKit/NSColor.h>
+#include <AppKit/NSGraphics.h>
 #include <AppKit/NSView.h>
 #include <AppKit/NSWindow.h>
 #include <GNUstepGUI/GSDisplayServer.h>
@@ -105,8 +106,8 @@ static NSMapTable *gtable;
 
 @interface GSContext (PrivateOps)
 - (void)DPSdefineuserobject;
-- (void)DPSexecuserobject: (int)index;
-- (void)DPSundefineuserobject: (int)index;
+- (void)DPSexecuserobject: (NSInteger)index;
+- (void)DPSundefineuserobject: (NSInteger)index;
 - (void)DPSclear;
 - (void)DPScopy: (int)n;
 - (void)DPScount: (int *)n;
@@ -116,6 +117,17 @@ static NSMapTable *gtable;
 - (void)DPSpop;
 @end
 
+@interface NSBitmapImageRep (GSPrivate)
+- (NSBitmapImageRep *) _convertToFormatBitsPerSample: (NSInteger)bps
+                                     samplesPerPixel: (NSInteger)spp
+                                            hasAlpha: (BOOL)alpha
+                                            isPlanar: (BOOL)isPlanar
+                                      colorSpaceName: (NSString*)colorSpaceName
+                                        bitmapFormat: (NSBitmapFormat)bitmapFormat 
+                                         bytesPerRow: (NSInteger)rowBytes
+                                        bitsPerPixel: (NSInteger)pixelBits;
+@end
+
 
 /**
    <unit>
@@ -135,8 +147,11 @@ static NSMapTable *gtable;
 
 + (void) initialize
 {
-  gtable = NSCreateMapTable(NSIntMapKeyCallBacks,
-                                 NSObjectMapValueCallBacks, 20);
+  if (self == [GSContext class])
+    {
+      gtable = NSCreateMapTable(NSIntMapKeyCallBacks,
+                                NSObjectMapValueCallBacks, 20);
+    }
 }
 
 + (void) insertObject: (id)obj forKey: (int)index
@@ -172,7 +187,7 @@ static NSMapTable *gtable;
 
   contextType = [info objectForKey: 
 		  NSGraphicsContextRepresentationFormatAttributeName];
-  if (([isa handlesPS] == NO) && contextType 
+  if (([object_getClass(self) handlesPS] == NO) && contextType 
       && [contextType isEqual: NSGraphicsContextPSFormat])
     {
       /* Don't call self, since we aren't initialized */
@@ -191,7 +206,7 @@ static NSMapTable *gtable;
       gstack =  NSZoneMalloc(z, sizeof(GSIArray_t));
       GSIArrayInitWithZoneAndCapacity((GSIArray)gstack, z, 2);
       /* Create a default gstate */
-      gstate = [[[isa GStateClass] allocWithZone: z] 
+      gstate = [[[object_getClass(self) GStateClass] allocWithZone: z] 
                    initWithDrawContext: self];
 
       // Special handling for window drawing
@@ -246,23 +261,23 @@ static NSMapTable *gtable;
 /* ----------------------------------------------------------------------- */
 /* Color operations */
 /* ----------------------------------------------------------------------- */
-- (void) DPScurrentalpha: (float *)a
+- (void) DPScurrentalpha: (CGFloat *)a
 {
   [gstate DPScurrentalpha: a];
 }
 
-- (void) DPScurrentcmykcolor: (float*)c : (float*)m : (float*)y : (float*)k 
+- (void) DPScurrentcmykcolor: (CGFloat*)c : (CGFloat*)m : (CGFloat*)y : (CGFloat*)k 
 {
   [gstate DPScurrentcmykcolor:c :m :y :k];
 }
 
-- (void) DPScurrentgray: (float*)gray 
+- (void) DPScurrentgray: (CGFloat*)gray 
 {
   CHECK_NULL_OUTPUT(gray);
   [gstate DPScurrentgray: gray];
 }
 
-- (void) DPScurrenthsbcolor: (float*)h : (float*)s : (float*)b 
+- (void) DPScurrenthsbcolor: (CGFloat*)h : (CGFloat*)s : (CGFloat*)b 
 {
   CHECK_NULL_OUTPUT(h);
   CHECK_NULL_OUTPUT(s);
@@ -270,7 +285,7 @@ static NSMapTable *gtable;
   [gstate DPScurrenthsbcolor:h :s :b];
 }
 
-- (void) DPScurrentrgbcolor: (float*)r : (float*)g : (float*)b 
+- (void) DPScurrentrgbcolor: (CGFloat*)r : (CGFloat*)g : (CGFloat*)b 
 {
   CHECK_NULL_OUTPUT(r);
   CHECK_NULL_OUTPUT(g);
@@ -278,27 +293,27 @@ static NSMapTable *gtable;
   [gstate DPScurrentrgbcolor:r :g :b];
 }
 
-- (void) DPSsetalpha: (float)a
+- (void) DPSsetalpha: (CGFloat)a
 {
   [gstate DPSsetalpha: a];
 }
 
-- (void) DPSsetcmykcolor: (float)c : (float)m : (float)y : (float)k 
+- (void) DPSsetcmykcolor: (CGFloat)c : (CGFloat)m : (CGFloat)y : (CGFloat)k 
 {
   [gstate DPSsetcmykcolor:c :m :y :k];
 }
 
-- (void) DPSsetgray: (float)gray 
+- (void) DPSsetgray: (CGFloat)gray 
 {
   [gstate DPSsetgray: gray];
 }
 
-- (void) DPSsethsbcolor: (float)h : (float)s : (float)b 
+- (void) DPSsethsbcolor: (CGFloat)h : (CGFloat)s : (CGFloat)b 
 {
   [gstate DPSsethsbcolor:h :s :b];
 }
 
-- (void) DPSsetrgbcolor: (float)r : (float)g : (float)b 
+- (void) DPSsetrgbcolor: (CGFloat)r : (CGFloat)g : (CGFloat)b 
 {
   [gstate DPSsetrgbcolor:r :g :b];
 }
@@ -331,12 +346,12 @@ static NSMapTable *gtable;
 /* ----------------------------------------------------------------------- */
 /* Text operations */
 /* ----------------------------------------------------------------------- */
-- (void) DPSashow: (float)x : (float)y : (const char *)s 
+- (void) DPSashow: (CGFloat)x : (CGFloat)y : (const char *)s 
 {
   [gstate DPSashow: x : y : s];
 }
 
-- (void) DPSawidthshow: (float)cx : (float)cy : (int)c : (float)ax : (float)ay : (const char *)s 
+- (void) DPSawidthshow: (CGFloat)cx : (CGFloat)cy : (int)c : (CGFloat)ax : (CGFloat)ay : (const char *)s 
 {
   [gstate DPSawidthshow: cx : cy : c : ax : ay : s];
 }
@@ -357,27 +372,27 @@ static NSMapTable *gtable;
   [gstate DPSshow: s];
 }
 
-- (void) DPSwidthshow: (float)x : (float)y : (int)c : (const char *)s 
+- (void) DPSwidthshow: (CGFloat)x : (CGFloat)y : (int)c : (const char *)s 
 {
   [gstate DPSwidthshow: x : y : c : s];
 }
 
-- (void) DPSxshow: (const char *)s : (const float*)numarray : (int)size 
+- (void) DPSxshow: (const char *)s : (const CGFloat*)numarray : (int)size 
 {
   [gstate DPSxshow: s : numarray : size];
 }
 
-- (void) DPSxyshow: (const char *)s : (const float*)numarray : (int)size 
+- (void) DPSxyshow: (const char *)s : (const CGFloat*)numarray : (int)size 
 {
   [gstate DPSxyshow: s : numarray : size];
 }
 
-- (void) DPSyshow: (const char *)s : (const float*)numarray : (int)size 
+- (void) DPSyshow: (const char *)s : (const CGFloat*)numarray : (int)size 
 {
   [gstate DPSyshow: s : numarray : size];
 }
 
-- (void) GSSetCharacterSpacing: (float)extra
+- (void) GSSetCharacterSpacing: (CGFloat)extra
 {
   [gstate GSSetCharacterSpacing: extra];
 }
@@ -387,7 +402,7 @@ static NSMapTable *gtable;
   [gstate GSSetFont: fontref];
 }
 
-- (void) GSSetFontSize: (float)size
+- (void) GSSetFontSize: (CGFloat)size
 {
   [gstate GSSetFontSize: size];
 }
@@ -458,7 +473,7 @@ static NSMapTable *gtable;
   [gstate DPSinitgraphics];
 }
 
-- (void) DPSsetgstate: (int)gst
+- (void) DPSsetgstate: (NSInteger)gst
 {
   if (gst)
     {
@@ -471,35 +486,35 @@ static NSMapTable *gtable;
     DESTROY(gstate);
 }
 
-- (int) GSDefineGState
+- (NSInteger) GSDefineGState
 {
   if (gstate == nil)
     {
       DPS_ERROR(DPSundefined, @"No gstate");
       return 0;
     }
-	[isa insertObject: AUTORELEASE([gstate copy]) forKey: ++unique_index];
+  [object_getClass(self) insertObject: AUTORELEASE([gstate copy]) forKey: ++unique_index];
 
   return unique_index;
 }
 
-- (void) GSUndefineGState: (int)gst
+- (void) GSUndefineGState: (NSInteger)gst
 {
   [self DPSundefineuserobject: gst];
 }
 
-- (void) GSReplaceGState: (int)gst
+- (void) GSReplaceGState: (NSInteger)gst
 {
   if (gst <= 0)
     return;
 
-	[isa insertObject: AUTORELEASE([gstate copy]) forKey: gst];
+  [object_getClass(self) insertObject: AUTORELEASE([gstate copy]) forKey: gst];
 }
 
 /* ----------------------------------------------------------------------- */
 /* Gstate operations */
 /* ----------------------------------------------------------------------- */
-- (void) DPScurrentflat: (float*)flatness
+- (void) DPScurrentflat: (CGFloat*)flatness
 {
   CHECK_NULL_OUTPUT(flatness);
   [gstate DPScurrentflat: flatness];
@@ -515,18 +530,18 @@ static NSMapTable *gtable;
   [gstate DPScurrentlinejoin: linejoin];
 }
 
-- (void) DPScurrentlinewidth: (float*)width
+- (void) DPScurrentlinewidth: (CGFloat*)width
 {
   [gstate DPScurrentlinewidth: width];
 }
 
-- (void) DPScurrentmiterlimit: (float*)limit
+- (void) DPScurrentmiterlimit: (CGFloat*)limit
 {
   CHECK_NULL_OUTPUT(limit);
   [gstate DPScurrentmiterlimit: limit];
 }
 
-- (void) DPScurrentpoint: (float*)x : (float*)y
+- (void) DPScurrentpoint: (CGFloat*)x : (CGFloat*)y
 {
   CHECK_NULL_OUTPUT(x);
   CHECK_NULL_OUTPUT(y);
@@ -539,17 +554,17 @@ static NSMapTable *gtable;
   [gstate DPScurrentstrokeadjust: b];
 }
 
-- (void) DPSsetdash: (const float*)pat : (int)size : (float)offset
+- (void) DPSsetdash: (const CGFloat*)pat : (NSInteger)size : (CGFloat)offset
 {
   [gstate DPSsetdash: pat : size : offset]; 
 }
 
-- (void) DPSsetflat: (float)flatness
+- (void) DPSsetflat: (CGFloat)flatness
 {
   [gstate DPSsetflat: flatness];
 }
 
-- (void) DPSsethalftonephase: (float)x : (float)y
+- (void) DPSsethalftonephase: (CGFloat)x : (CGFloat)y
 {
   [self notImplemented: _cmd];
 }
@@ -564,12 +579,12 @@ static NSMapTable *gtable;
   [gstate DPSsetlinejoin: linejoin];
 }
 
-- (void) DPSsetlinewidth: (float)width
+- (void) DPSsetlinewidth: (CGFloat)width
 {
   [gstate DPSsetlinewidth: width];
 }
 
-- (void) DPSsetmiterlimit: (float)limit
+- (void) DPSsetmiterlimit: (CGFloat)limit
 {
   [gstate DPSsetmiterlimit: limit];
 }
@@ -582,7 +597,7 @@ static NSMapTable *gtable;
 /* ----------------------------------------------------------------------- */
 /* Matrix operations */
 /* ----------------------------------------------------------------------- */
-- (void) DPSconcat: (const float*)m
+- (void) DPSconcat: (const CGFloat*)m
 {
   [gstate DPSconcat: m];
 }
@@ -592,17 +607,17 @@ static NSMapTable *gtable;
   [gstate DPSinitmatrix];
 }
 
-- (void) DPSrotate: (float)angle
+- (void) DPSrotate: (CGFloat)angle
 {
   [gstate DPSrotate: angle];
 }
 
-- (void) DPSscale: (float)x : (float)y
+- (void) DPSscale: (CGFloat)x : (CGFloat)y
 {
   [gstate DPSscale:x :y];
 }
 
-- (void) DPStranslate: (float)x : (float)y
+- (void) DPStranslate: (CGFloat)x : (CGFloat)y
 {
   [gstate DPStranslate:x :y];
 }
@@ -625,19 +640,19 @@ static NSMapTable *gtable;
 /* ----------------------------------------------------------------------- */
 /* Paint operations */
 /* ----------------------------------------------------------------------- */
-- (void) DPSarc: (float)x : (float)y : (float)r : (float)angle1 
-	       : (float)angle2
+- (void) DPSarc: (CGFloat)x : (CGFloat)y : (CGFloat)r : (CGFloat)angle1 
+	       : (CGFloat)angle2
 {
   [gstate DPSarc: x : y : r : angle1 : angle2];
 }
 
-- (void) DPSarcn: (float)x : (float)y : (float)r : (float)angle1 
-		: (float)angle2
+- (void) DPSarcn: (CGFloat)x : (CGFloat)y : (CGFloat)r : (CGFloat)angle1 
+		: (CGFloat)angle2
 {
   [gstate DPSarcn: x : y : r : angle1 : angle2];
 }
 
-- (void) DPSarct: (float)x1 : (float)y1 : (float)x2 : (float)y2 : (float)r;
+- (void) DPSarct: (CGFloat)x1 : (CGFloat)y1 : (CGFloat)x2 : (CGFloat)y2 : (CGFloat)r;
 {
   [gstate DPSarct: x1 : y1 : x2 : y2 : r];
 }
@@ -652,8 +667,8 @@ static NSMapTable *gtable;
   [gstate DPSclosepath];
 }
 
-- (void) DPScurveto: (float)x1 : (float)y1 : (float)x2 : (float)y2 
-		   : (float)x3 : (float)y3
+- (void) DPScurveto: (CGFloat)x1 : (CGFloat)y1 : (CGFloat)x2 : (CGFloat)y2 
+		   : (CGFloat)x3 : (CGFloat)y3
 {
   [gstate DPScurveto: x1 : y1 : x2 : y2 : x3 : y3];
 }
@@ -683,12 +698,12 @@ static NSMapTable *gtable;
   [gstate DPSinitclip];
 }
 
-- (void) DPSlineto: (float)x : (float)y
+- (void) DPSlineto: (CGFloat)x : (CGFloat)y
 {
   [gstate DPSlineto: x : y];
 }
 
-- (void) DPSmoveto: (float)x : (float)y
+- (void) DPSmoveto: (CGFloat)x : (CGFloat)y
 {
   [gstate DPSmoveto: x : y];
 }
@@ -698,28 +713,28 @@ static NSMapTable *gtable;
   [gstate DPSnewpath];
 }
 
-- (void) DPSpathbbox: (float*)llx : (float*)lly : (float*)urx : (float*)ury
+- (void) DPSpathbbox: (CGFloat*)llx : (CGFloat*)lly : (CGFloat*)urx : (CGFloat*)ury
 {
   [gstate DPSpathbbox: llx : lly : urx : ury];
 }
 
-- (void) DPSrcurveto: (float)x1 : (float)y1 : (float)x2 : (float)y2 
-		    : (float)x3 : (float)y3
+- (void) DPSrcurveto: (CGFloat)x1 : (CGFloat)y1 : (CGFloat)x2 : (CGFloat)y2 
+		    : (CGFloat)x3 : (CGFloat)y3
 {
   [gstate DPSrcurveto: x1 : y1 : x2 : y2 : x3 : y3];
 }
 
-- (void) DPSrectclip: (float)x : (float)y : (float)w : (float)h
+- (void) DPSrectclip: (CGFloat)x : (CGFloat)y : (CGFloat)w : (CGFloat)h
 {
   [gstate DPSrectclip: x : y : w : h];
 }
 
-- (void) DPSrectfill: (float)x : (float)y : (float)w : (float)h
+- (void) DPSrectfill: (CGFloat)x : (CGFloat)y : (CGFloat)w : (CGFloat)h
 {
   [gstate DPSrectfill:x :y :w :h];
 }
 
-- (void) DPSrectstroke: (float)x : (float)y : (float)w : (float)h
+- (void) DPSrectstroke: (CGFloat)x : (CGFloat)y : (CGFloat)w : (CGFloat)h
 {
   [gstate DPSrectstroke:x :y :w :h];
 }
@@ -729,12 +744,12 @@ static NSMapTable *gtable;
   [gstate DPSreversepath];
 }
 
-- (void) DPSrlineto: (float)x : (float)y
+- (void) DPSrlineto: (CGFloat)x : (CGFloat)y
 {
   [gstate DPSrlineto: x : y];
 }
 
-- (void) DPSrmoveto: (float)x : (float)y
+- (void) DPSrmoveto: (CGFloat)x : (CGFloat)y
 {
   [gstate DPSrmoveto: x : y];
 }
@@ -780,8 +795,8 @@ static NSMapTable *gtable;
 /*-------------------------------------------------------------------------*/
 /* Graphics Extension Ops */
 /*-------------------------------------------------------------------------*/
-- (void) DPScomposite: (float)x : (float)y : (float)w : (float)h 
-		     : (int)gstateNum : (float)dx : (float)dy : (int)op
+- (void) DPScomposite: (CGFloat)x : (CGFloat)y : (CGFloat)w : (CGFloat)h 
+		     : (NSInteger)gstateNum : (CGFloat)dx : (CGFloat)dy : (NSCompositingOperation)op
 {
   [self GScomposite: gstateNum
         toPoint: NSMakePoint(dx, dy)
@@ -790,13 +805,13 @@ static NSMapTable *gtable;
         fraction: 1.0];
 }
 
-- (void) DPScompositerect: (float)x : (float)y : (float)w : (float)h : (int)op
+- (void) DPScompositerect: (CGFloat)x : (CGFloat)y : (CGFloat)w : (CGFloat)h : (NSCompositingOperation)op
 {
-  [gstate  compositerect: NSMakeRect(x, y, w, h) op: op];
+  [gstate compositerect: NSMakeRect(x, y, w, h) op: op];
 }
 
-- (void) DPSdissolve: (float)x : (float)y : (float)w : (float)h 
-		    : (int)gstateNum : (float)dx : (float)dy : (float)delta
+- (void) DPSdissolve: (CGFloat)x : (CGFloat)y : (CGFloat)w : (CGFloat)h 
+		    : (NSInteger)gstateNum : (CGFloat)dx : (CGFloat)dy : (CGFloat)delta
 {
   [self GScomposite: gstateNum
         toPoint: NSMakePoint(dx, dy)
@@ -805,11 +820,11 @@ static NSMapTable *gtable;
         fraction: delta];
 }
 
-- (void) GScomposite: (int)gstateNum
+- (void) GScomposite: (NSInteger)gstateNum
 	     toPoint: (NSPoint)aPoint
 	    fromRect: (NSRect)srcRect
 	   operation: (NSCompositingOperation)op
-	    fraction: (float)delta
+	    fraction: (CGFloat)delta
 {
   GSGState *g = gstate;
 
@@ -826,11 +841,11 @@ static NSMapTable *gtable;
                  fraction: delta];
 }
 
-- (void) GSdraw: (int)gstateNum
+- (void) GSdraw: (NSInteger)gstateNum
 	toPoint: (NSPoint)aPoint
        fromRect: (NSRect)srcRect
       operation: (NSCompositingOperation)op
-       fraction: (float)delta
+       fraction: (CGFloat)delta
 {
   GSGState *g = gstate;
 
@@ -847,12 +862,42 @@ static NSMapTable *gtable;
             fraction: delta];
 }
 
-- (void) GSDrawImage: (NSRect) rect: (void *) imageref
+- (BOOL) isCompatibleBitmap: (NSBitmapImageRep*)bitmap
+{
+  return ([bitmap bitmapFormat] == 0);
+}
+
+- (void) GSDrawImage: (NSRect) rect : (void *) imageref
 {
   NSBitmapImageRep *bitmap;
   unsigned char *data[5];
 
   bitmap = (NSBitmapImageRep*)imageref;
+  if (![self isCompatibleBitmap: bitmap])
+    {
+      NSInteger bitsPerSample = 8;
+      BOOL isPlanar = NO;
+      NSInteger samplesPerPixel = [bitmap hasAlpha] ? 4 : 3;
+      NSString *colorSpaceName = NSCalibratedRGBColorSpace;
+      NSBitmapImageRep *new;
+
+     new = [bitmap _convertToFormatBitsPerSample: bitsPerSample
+                    samplesPerPixel: samplesPerPixel
+                    hasAlpha: [bitmap hasAlpha]
+                    isPlanar: isPlanar
+                    colorSpaceName: colorSpaceName
+                    bitmapFormat: 0
+                    bytesPerRow: 0
+                    bitsPerPixel: 0];
+            
+      if (new == nil)
+        {
+          NSLog(@"Could not convert bitmap data");
+          return;
+        }
+      bitmap = new;
+    }
+
   [bitmap getBitmapDataPlanes: data];
   [self NSDrawBitmap: rect : [bitmap pixelsWide] : [bitmap pixelsHigh]
         : [bitmap bitsPerSample] : [bitmap samplesPerPixel]
@@ -889,9 +934,9 @@ static NSMapTable *gtable;
 /*
  * Render Bitmap Images
  */
-- (void) NSDrawBitmap: (NSRect) rect : (int) pixelsWide : (int) pixelsHigh
-		     : (int) bitsPerSample : (int) samplesPerPixel 
-		     : (int) bitsPerPixel : (int) bytesPerRow : (BOOL) isPlanar
+- (void) NSDrawBitmap: (NSRect) rect : (NSInteger) pixelsWide : (NSInteger) pixelsHigh
+		     : (NSInteger) bitsPerSample : (NSInteger) samplesPerPixel 
+		     : (NSInteger) bitsPerPixel : (NSInteger) bytesPerRow : (BOOL) isPlanar
 		     : (BOOL) hasAlpha : (NSString *) colorSpaceName
 		     : (const unsigned char *const [5]) data
 {
@@ -936,20 +981,21 @@ static NSMapTable *gtable;
   int n;
   id obj;
   NSNumber *number;
+
   ctxt_pop(obj, opstack, NSObject);
   ctxt_pop(number, opstack, NSNumber);
   n = [number intValue];
   if (n < 0)
     DPS_ERROR(DPSinvalidparam, @"Invalid userobject index");
   else 
-		[isa insertObject: obj forKey: n];
+    [object_getClass(self) insertObject: obj forKey: n];
 }
 
-- (void)DPSexecuserobject: (int)index
+- (void)DPSexecuserobject: (NSInteger)index
 {
   id obj;
 
-  if (index < 0 || (obj = [isa getObjectForKey: index]) == nil)
+  if (index < 0 || (obj = [object_getClass(self) getObjectForKey: index]) == nil)
     {
       DPS_ERROR(DPSinvalidparam, @"Invalid userobject index");
       return;
@@ -957,14 +1003,14 @@ static NSMapTable *gtable;
   ctxt_push(obj, opstack);
 }
 
-- (void)DPSundefineuserobject: (int)index
+- (void)DPSundefineuserobject: (NSInteger)index
 {
-  if (index < 0 || [isa getObjectForKey: index] == nil)
+  if (index < 0 || [object_getClass(self) getObjectForKey: index] == nil)
     {
       DPS_ERROR(DPSinvalidparam, @"Invalid gstate index");
       return;
     }
-  [isa removeObjectForKey: index];
+  [object_getClass(self) removeObjectForKey: index];
 }
 
 - (void)DPSclear 
