@@ -890,10 +890,12 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
              */
             if (!NSEqualSizes(r.size, x.size))
               {
+		NSEvent *r;
+
                 /* Resize events move the origin. There's no good
                    place to pass this info back, so we put it in
                    the event location field */
-                e = [NSEvent otherEventWithType: NSAppKitDefined
+                r = [NSEvent otherEventWithType: NSAppKitDefined
                              location: n.origin
                              modifierFlags: eventFlags
                              timestamp: ts / 1000.0
@@ -902,14 +904,19 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
                              subtype: GSAppKitWindowResized
                              data1: n.size.width
                              data2: n.size.height];
+
+		/* We don't add this event in event_queue, to don't delay
+		 * its sent. Instead, send it directly to the window. If not,
+		 * the programa can move/resize the window while we send
+		 * this event, causing a confusion.
+		 */
+		[[NSApp windowWithWindowNumber: cWin->number] sendEvent: r];
               }
             if (!NSEqualPoints(r.origin, x.origin))
               {
-                if (e != nil)
-                  {
-                    [event_queue addObject: e];
-                  }
-                e = [NSEvent otherEventWithType: NSAppKitDefined
+		NSEvent *r;
+
+                r = [NSEvent otherEventWithType: NSAppKitDefined
                              location: eventLocation
                              modifierFlags: eventFlags
                              timestamp: ts / 1000.0
@@ -918,6 +925,13 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
                              subtype: GSAppKitWindowMoved
                              data1: n.origin.x
                              data2: n.origin.y];
+
+		/* We don't add this event in event_queue, to don't delay
+		 * its sent. Instead, send it directly to the window. If not,
+		 * the programa can move/resize the window while we send
+		 * this event, causing a confusion.
+		 */
+		[[NSApp windowWithWindowNumber: cWin->number] sendEvent: r];
               }
           }	
         break;
@@ -1934,7 +1948,7 @@ initialize_keyboard (void)
 
   
   set_up_num_lock ();
-  _mod_ignore_shift = [defaults boolForKey: @"GSModifiersAreKeys"];
+  _mod_ignore_shift = ![defaults boolForKey: @"GSModifiersAreNotKeys"];
   
   _is_keyboard_initialized = YES;
 }
@@ -2050,7 +2064,7 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType,
      XIM events can potentially return this. */
   /* Possibly ignore shift/other modifier state in determining KeySym to
      work around correct but undesired behavior with shifted modifiers.
-     See Back defaults documentation for "GSModifiersAreKeys". */
+     See Back defaults documentation for "GSModifiersAreNotKeys". */
   modKeysym = (_mod_ignore_shift == YES) ?
       XLookupKeysym((XKeyEvent *)xEvent, 0) : keysym;
   if (modKeysym != NoSymbol)
